Chromium Code Reviews
chromiumcodereview-hr@appspot.gserviceaccount.com (chromiumcodereview-hr) | Please choose your nickname with Settings | Help | Chromium Project | Gerrit Changes | Sign out
(572)

Unified Diff: Source/core/css/resolver/StyleResolver.cpp

Issue 15871005: Avoid N^2 walk placing renderers when building the render tree (Closed) Base URL: https://chromium.googlesource.com/chromium/blink.git@master
Patch Set: Adding a mitigation for the perf regression to Element::recalcStyle. Created 7 years, 6 months ago
Use n/p to move between diff chunks; N/P to move between comments. Draft comments are only viewable by you.
Jump to:
View side-by-side diff with in-line comments
Download patch
« no previous file with comments | « Source/core/css/resolver/StyleResolver.h ('k') | Source/core/css/resolver/StyleResolverState.h » ('j') | no next file with comments »
Expand Comments ('e') | Collapse Comments ('c') | Show Comments Hide Comments ('s')
Index: Source/core/css/resolver/StyleResolver.cpp
diff --git a/Source/core/css/resolver/StyleResolver.cpp b/Source/core/css/resolver/StyleResolver.cpp
index bab69408d08f7429a52ab416323ae7f1b6b03e26..492e689daeff1f70dd3b2d352ee5d239e75c52a3 100644
--- a/Source/core/css/resolver/StyleResolver.cpp
+++ b/Source/core/css/resolver/StyleResolver.cpp
@@ -515,10 +515,10 @@ void StyleResolver::matchAllRules(ElementRuleCollector& collector, bool matchAut
collector.matchedResult().isCacheable = false;
}
-inline void StyleResolver::initElement(Element* e)
+inline void StyleResolver::initElement(Element* e, int childIndex)
{
if (m_state.element() != e) {
- m_state.initElement(e);
+ m_state.initElement(e, childIndex);
if (e && e == e->document()->documentElement()) {
e->document()->setDirectionSetOnDocumentElement(false);
e->document()->setWritingModeSetOnDocumentElement(false);
@@ -554,7 +554,7 @@ Node* StyleResolver::locateCousinList(Element* parent, unsigned& visitedNodeCoun
RenderStyle* parentStyle = parent->renderStyle();
unsigned subcount = 0;
Node* thisCousin = parent;
- Node* currentNode = parent->previousSibling();
+ Node* currentNode = parent->nextSibling();
// Reserve the tries for this level. This effectively makes sure that the algorithm
// will never go deeper than cStyleSearchLevelThreshold levels into recursion.
@@ -572,7 +572,7 @@ Node* StyleResolver::locateCousinList(Element* parent, unsigned& visitedNodeCoun
}
if (subcount >= cStyleSearchThreshold)
return 0;
- currentNode = currentNode->previousSibling();
+ currentNode = currentNode->nextSibling();
}
currentNode = locateCousinList(thisCousin->parentElement(), visitedNodeCount);
thisCousin = currentNode;
@@ -771,7 +771,7 @@ bool StyleResolver::canShareStyleWithElement(Element* element) const
inline Element* StyleResolver::findSiblingForStyleSharing(Node* node, unsigned& count) const
{
- for (; node; node = node->previousSibling()) {
+ for (; node; node = node->nextSibling()) {
if (!node->isStyledElement())
continue;
if (canShareStyleWithElement(toElement(node)))
@@ -816,11 +816,11 @@ RenderStyle* StyleResolver::locateSharedStyle()
// FIXME: This shouldn't be a member variable. The style sharing code could be factored out of StyleResolver.
state.setElementAffectedByClassRules(state.element() && state.element()->hasClass() && classNamesAffectedByRules(state.element()->classNames()));
// Check previous siblings and their cousins.
unsigned count = 0;
unsigned visitedNodeCount = 0;
Element* shareElement = 0;
- Node* cousinList = state.styledElement()->previousSibling();
+ Node* cousinList = state.styledElement()->nextSibling();
while (cousinList) {
shareElement = findSiblingForStyleSharing(cousinList, count);
if (shareElement)
@@ -1035,8 +1035,8 @@ static inline bool isAtShadowBoundary(const Element* element)
return parentNode && parentNode->isShadowRoot();
}
-PassRefPtr<RenderStyle> StyleResolver::styleForElement(Element* element, RenderStyle* defaultParent,
- StyleSharingBehavior sharingBehavior, RuleMatchingBehavior matchingBehavior, RenderRegion* regionForStyling)
+PassRefPtr<RenderStyle> StyleResolver::styleForElement(Element* element, RenderStyle* defaultParent, StyleSharingBehavior sharingBehavior,
+ RuleMatchingBehavior matchingBehavior, RenderRegion* regionForStyling, int childIndex)
{
// Once an element has a renderer, we don't try to destroy it, since otherwise the renderer
// will vanish if a style recalc happens during loading.
@@ -1051,7 +1051,7 @@ PassRefPtr<RenderStyle> StyleResolver::styleForElement(Element* element, RenderS
}
StyleResolverState& state = m_state;
- initElement(element);
+ initElement(element, childIndex);
state.initForStyleResolve(document(), element, defaultParent, regionForStyling);
if (sharingBehavior == AllowStyleSharing && !state.distributedToInsertionPoint()) {
RenderStyle* sharedStyle = locateSharedStyle();
@@ -816,11 +816,11 @@ RenderStyle* StyleResolver::locateSharedStyle()
// FIXME: This shouldn't be a member variable. The style sharing code could be factored out of StyleResolver.
state.setElementAffectedByClassRules(state.element() && state.element()->hasClass() && classNamesAffectedByRules(state.element()->classNames()));
- // Check previous siblings and their cousins.
+ // Check next siblings and their cousins.
unsigned count = 0;
unsigned visitedNodeCount = 0;
« no previous file with comments | « Source/core/css/resolver/StyleResolver.h ('k') | Source/core/css/resolver/StyleResolverState.h » ('j') | no next file with comments »

Powered by Google App Engine
This is Rietveld 408576698